package org.zeroturnaround.javarebel.integration.resin;

import org.zeroturnaround.bundled.javassist.ClassPool;
import org.zeroturnaround.bundled.javassist.CtClass;
import org.zeroturnaround.bundled.javassist.CtConstructor;
import org.zeroturnaround.bundled.javassist.CtMethod;
import org.zeroturnaround.bundled.javassist.CtNewMethod;
import org.zeroturnaround.bundled.javassist.NotFoundException;
import org.zeroturnaround.javarebel.integration.support.JavassistClassBytecodeProcessor;

/**
 * Transforms <code>com.caucho.loader.DynamicClassLoader</code>.
 * 
 * <p>Tested on Resin 3.0.27.</p>
 * <p>Resource providing methods should work on 3.1.8 and 3.2.1 also.</p>
 * 
 * @author Rein Raudjärv
 */
public class DynamicClassLoaderCBP extends JavassistClassBytecodeProcessor {

  public void process(ClassPool cp, ClassLoader cl, CtClass ctClass) throws Exception {
    cp.importPackage("java.io");
    cp.importPackage("java.net");
    cp.importPackage("java.util");
    cp.importPackage("com.caucho.loader");
    cp.importPackage("org.zeroturnaround.javarebel");
    
    CtConstructor[] constructors = ctClass.getConstructors();
    for (int i = 0; i < constructors.length; i++) {
      CtConstructor constructor = constructors[i];
      constructor.insertAfter(
          "org.zeroturnaround.javarebel.IntegrationFactory.getInstance()" +
          "  .registerClassLoader($0, " +
          "    new org.zeroturnaround.javarebel.integration.generic.FindResourceClassResourceSource($0));");
    }

    {
      CtMethod m = ctClass.getMethod("findClass", "(Ljava/lang/String;)Ljava/lang/Class;");
      m.insertBefore(
          "{ synchronized ($0) {" + 
          "    Class result =" + 
          "      findLoadedClass($1);" + 
          "    if (result != null)" + 
          "      return result;" + 
          "    result = " + 
          "      org.zeroturnaround.javarebel.IntegrationFactory.getInstance()" + 
          "        .findReloadableClass($0, $1);" + 
          "    if (result != null)" + 
          "      return result;" + 
          "}}");
    }
    
    try {
      // getImportResource was added in Resin 3.2 for resources from OSGI
      ctClass.getDeclaredMethod("getImportResource");
    } catch (NotFoundException e) {
      ctClass.addMethod(CtNewMethod.make(
          "protected URL getImportResource(String name) {" +
          "  return null;" +
          "}", ctClass));
    }
    
    ctClass.addMethod(CtNewMethod.make(
        "private URL __getLocalResource(String name) {" +

        "  Integration integration = IntegrationFactory.getInstance();" +
        "  if (integration.isResourceReplaced($0, name))" +
        "    return integration.findResource($0, name);" +
        
        "  URL url = getImportResource(name);" +
        "  if (url != null)" +
        "    return url;" +

        "  ArrayList loaders = getLoaders();" +
        "  for (int i = 0; loaders != null && i < loaders.size(); i++) {" +
        "    Loader loader = (Loader) loaders.get(i);" +
        "    url = loader.getResource(name);" +
        "    if (url != null)" +
        "      return url;" +
        "  }" +

        "  return null;" +

        "}", ctClass));
    
    ctClass.addMethod(CtNewMethod.make(
        "private InputStream __getLocalResourceAsStream(String name) {" +

        "  Integration integration = IntegrationFactory.getInstance();" +
        "  if (integration.isResourceReplaced($0, name)) {" +
        "    URL url = integration.findResource($0, name);" +

        "    if (url == null)" +
        "      return null;" +
        
        "    try {" +
        "      return url.openStream();" +
        "    } catch (IOException e) {" +
        "      return null;" +
        "    }" +
        "  }" +
        
        "  ArrayList loaders = getLoaders();" +
        "  for (int i = 0; loaders != null && i < loaders.size(); i++) {" +
        "    Loader loader = (Loader) loaders.get(i);" +
        "    InputStream is = loader.getResourceAsStream(name);" +
        "    if (is != null)" +
        "      return is;" +
        "  }" +

        "  return null;" +

        "}", ctClass));    

    try {
      CtMethod m = ctClass.getDeclaredMethod("getResource");
      m.setBody("{" +
          "  if ($1.startsWith(\"/\"))" +
          "    $1 = $1.substring(1);" +

          "  if ($1.endsWith(\"/\"))" +
          "    $1 = $1.substring(0, $1.length() - 1);" +

          "  boolean isNormalJdkOrder = isNormalJdkOrder($1);" +
          "  URL url;" +
          
          "  if (isNormalJdkOrder) {" +
          "    url = getParentResource($1);" +
          "    if (url != null)" +
          "      return url;" +
          "  }" +
          
          "  url = __getLocalResource($1);" +
          "  if (url != null)" +
          "    return url;" +

          "  if (! isNormalJdkOrder) {" +
          "    url = getParentResource($1);" +
          "    if (url != null)" +
          "      return url;" +
          "  }" +

          "  return null;" +
          "}");
    }
    catch (NotFoundException e) {
      // skip
    }
    
    try {
      CtMethod m = ctClass.getDeclaredMethod("getResourceAsStream");
      m.setBody("{" +
          "  if ($1.startsWith(\"/\"))" +
          "    $1 = $1.substring(1);" +

          "  if ($1.endsWith(\"/\"))" +
          "    $1 = $1.substring(0, $1.length() - 1);" +
          
          "  boolean isNormalJdkOrder = isNormalJdkOrder($1);" +
          "  InputStream is;" +
          
          "  if (isNormalJdkOrder) {" +
          "    is = getParentResourceAsStream($1);" +
          "    if (is != null)" +
          "      return is;" +
          "  }" +
          
          "  is = __getLocalResourceAsStream($1);" +
          "  if (is != null)" +
          "    return is;" +
          
          "  if (! isNormalJdkOrder) {" +
          "    is = getParentResourceAsStream($1);" +
          "    if (is != null)" +
          "      return is;" +
          "  }" +
          
          "  return null;" +
      "}");
    }
    catch (NotFoundException e) {
      // skip
    }

    try {
      CtMethod m = ctClass.getDeclaredMethod("findResources", cp.get(new String[] {String.class.getName()}));
      m.insertBefore(
          "{" + 

          "  String name = $1;" +
          "  if (name.startsWith(\"/\"))" +
          "    name = name.substring(1);" +

          "  Integration integration = IntegrationFactory.getInstance();" +
          "  if (integration.isResourceReplaced($0, name))" +
          "    return integration.findResources($0, name);" + 

          "}");
    }
    catch (NotFoundException e) {
      // skip
    }

  }
}
